<?php

/**
 * Expose menu items as context reactions.
 */
class context_reaction_alter_form extends context_reaction {
  /**
   * Provide a form elements allowing adding fields and values.
   */
  function options_form($context) {
    // Initalize.
    $form = array();
    $delta = 0;
    $notes = t('<h3>Alter Forms</h3>
      <ul>
        <li>Parent heirarchy should be listed including the target element (comma separated), eg: "group_THIS, field_THIS".</li>
        <li>Attributes should use Drupal form API norms, eg: #default_value, #title, #maxlength. See !api_link.</li>
        <li>When appended, the alteration will detect array or string data type.</li>
        <li>You can also leave parent empty to modify the main form array. Append is assumed in this case.</li>
      </ul>',
      array('!api_link' => l(
        t('Form API Reference'),
        'http://api.drupal.org/api/drupal/developer%21topics%21forms_api_reference.html/7'
      ))
    );
    $form['elements'] = array(
      '#theme' => 'context_form_alteration_settings_table',
      '#prefix' => '<div id="context-form-alteration-elements-table">' . $notes,
      '#suffix' => '</div>',
    );

    // Build existing entries.
    $values = $this->fetch_from_context($context);
    foreach ($values['elements'] as $val) {
      $form['elements'][$delta] = $this->build_row($delta, $val['form_id'], $val['parents'], $val['attribute'], $val['value'], $val['append']);
      $delta++;
    }

    // Add placeholder rows
    $number_of_empty_rows = 3;
    for ($i = 0; $i < $number_of_empty_rows; $i++) {
      $form['elements'][$delta] = $this->build_row($delta);
      $delta++;
    }

    return $form;
  }

  /**
   * Override of options_form_submit().
   */
  function options_form_submit($values) {    
    // Clean up empty values for storage in neat, tidy array.
    foreach ($values['elements'] as $key => $option)  {
      if($option['form_id'] == '' || $option['parents'] == '' || $option['attribute'] == '') {
        unset($values['elements'][$key]);
      }
    }
    return $values;
  }

  function execute(&$form) {
    $contexts = $this->get_contexts();
    foreach ($contexts as $context) {
      if (!empty($context->reactions[$this->plugin])) {
        foreach ($context->reactions[$this->plugin] as $option_id => $option) {

          // Whether we should bother.
          foreach ($option as $opt) {            
            if ($opt['form_id'] === $form['#form_id']) {
              // Alter the form value.
              // @todo Add option for forcing.
              if ($opt['parents']) {
                $parents = array();
                $parents = explode(',', $opt['parents']);
                $parents[] = $opt['attribute'];
              }
              else {
                $parents = array($opt['attribute']);
              } 
              // Trim up the values, allowing messier/easier user input.
              array_walk($parents, create_function('&$val', '$val = trim($val);'));
              $this->craf_drupal_array_set_nested_value($form, $parents, $opt['value'], FALSE, $opt['append']);
            }
          }

        }
      }
    }
  }

  /**
  * Build a form row.
  */
  protected function build_row($delta, $form_id = NULL, $parents = NULL, $attribute = NULL, $value = NULL, $append = 0) {
    $form['form_id'] = array(
      '#default_value' => $form_id,
      '#type' => 'textfield',
      '#size' => 20,
    );
    $form['parents'] = array(
      '#default_value' => $parents,
      '#type' => 'textfield',
      '#size' => 20,
    );
    $form['attribute'] = array(
      '#default_value' => $attribute,
      '#type' => 'textfield',
      '#size' => 20,
    );
    $form['value'] = array(
      '#default_value' => $value,
      '#type' => 'textfield',
      '#size' => 20,
    );
    $form['append'] = array(
      '#default_value' => $append,
      '#type' => 'checkbox',
      '#size' => 20,
    );
    return $form;
  }

  /**
   * Override of drupal_array_set_nested_value(), allows append.
   */
  protected function craf_drupal_array_set_nested_value(array &$array, array $parents, $value, $force = FALSE, $append = FALSE) {
    $ref = &$array;
    foreach ($parents as $parent) {
      // PHP auto-creates container arrays and NULL entries without error if $ref
      // is NULL, but throws an error if $ref is set, but not an array.
      if ($force && isset($ref) && !is_array($ref)) {
        $ref = array();
      }
      $ref = &$ref[$parent];
    }
    if ($append) {
      if (is_array($ref)) {
        $ref[] = $value;
        return;
      }
      else {
        $ref .= $value;
      }
    }
    else {
      $ref = $value;
    }
  }
}
